@@ -4,18 +4,18 @@ use warnings;
 use strict;
 use 5.010001;
 
-our $VERSION = '0.021';
+our $VERSION = '0.050';
 use Exporter 'import';
 our @EXPORT_OK = qw( choose );
 
 use Parse::ANSIColor::Tiny qw();
 use Term::ANSIColor        qw( colored );
 use Text::ANSI::WideUtil   qw( ta_mbtrunc );
-use Unicode::GCString      qw();
 
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use Term::Choose::Constants qw( :choose :linux );
+use Term::Choose::LineFold qw( print_columns );
 
 use parent 'Term::Choose';
 
@@ -49,30 +49,24 @@ sub choose {
 
 sub __copy_orig_list {
     my ( $self ) = @_;
+    $self->{list} = [ @{$self->{orig_list}} ];
     if ( $self->{ll} ) {
-        $self->{list} = [ map {
-            my $copy = $_;
-            if ( ! $copy ) {
-                $copy = $self->{undef} if ! defined $copy;
-                $copy = $self->{empty} if $copy eq '';
-            }
-            $copy;
-        } @{$self->{orig_list}} ];
+        for ( @{$self->{list}} ) {
+            $_ = $self->{undef} if ! defined $_;
+        }
     }
     else {
-        $self->{list} = [ map {
-            my $copy = $_;
-            if ( ! $copy ) {
-                $copy = $self->{undef} if ! defined $copy;
-                $copy = $self->{empty} if $copy eq '';
+        for ( @{$self->{list}} ) {
+            if ( ! $_ ) {
+                $_ = $self->{undef} if ! defined $_;
+                $_ = $self->{empty} if $_ eq '';
             }
-            if ( ref $copy ) {
-                $copy = sprintf "%s(0x%x)", ref $copy, $copy;
+            if ( ref ) {
+                $_ = sprintf "%s(0x%x)", ref $_, $_;
             }
-            $copy =~ s/\p{Space}/ /g;               # replace, but don't squash sequences of spaces
-            $copy =~ s/\p{C}/$&=~m|\e| && $&/eg;    # remove \p{C} but keep \e
-            $copy;
-        } @{$self->{orig_list}} ];
+            s/\p{Space}/ /g;               # replace, but don't squash sequences of spaces
+            s/\p{C}/$&=~m|\e| && $&/eg;    # remove \p{C} but keep \e
+        }
     }
 }
 
@@ -82,15 +76,14 @@ sub __unicode_trim {
     return ta_mbtrunc( $str, $len );
 }
 
+sub _strip_ansi_color {
+    ( my $str = $_[0] ) =~ s/\e\[[\d;]*m//msg;
+    return $str;
+}
 
 sub __print_columns {
     #my $self = $_[0];
-    Unicode::GCString->new( __strip_ansi_color( $_[1] ) )->columns();
-}
-
-sub __strip_ansi_color {
-    ( my $str = $_[0] ) =~ s/\e\[[\d;]*m//msg;
-    return $str;
+    print_columns( _strip_ansi_color( $_[1] ) );
 }
 
 
@@ -192,7 +185,7 @@ Term::Choose_HAE - Choose items from a list interactively.
 
 =head1 VERSION
 
-Version 0.021
+Version 0.050
 
 =cut
 
@@ -315,7 +308,7 @@ L<stackoverflow|http://stackoverflow.com> for the help.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (C) 2015 Matthäus Kiem.
+Copyright (C) 2015-2016 Matthäus Kiem.
 
 This library is free software; you can redistribute it and/or modify it under the same terms as Perl 5.10.0. For
 details, see the full text of the licenses in the file LICENSE.
